@@ -28,13 +28,13 @@
 # include	<X11/StringDefs.h>
 # include	<X11/Shell.h>
 # include	<X11/Xos.h>
-# include	<X11/Xaw/Command.h>
+# include	<Xkw/KCommand.h>
 # include	<X11/Xaw/Box.h>
 # include	<X11/Xaw/Dialog.h>
-# include	<X11/Xaw/Label.h>
-# include	<X11/Xaw/MenuButton.h>
-# include	<X11/Xaw/SimpleMenu.h>
-# include	<X11/Xaw/SmeBSB.h>
+# include	<Xkw/KLabel.h>
+# include	<Xkw/KMenuButton.h>
+# include	<Xkw/KSimpleMenu.h>
+# include	<Xkw/KSmeBSB.h>
 # include	<X11/Xaw/AsciiText.h>
 # include	<X11/Xaw/Cardinals.h>
 # include	<Xkw/Cards.h>
@@ -42,6 +42,7 @@
 # include	<X11/Xutil.h>
 # include	<Xkw/CardsUtil.h>
 # include	<Xkw/Message.h>
+# include	"KKlondike-res.h"
 
 Widget	    toplevel;
 Widget	    frame;
@@ -69,8 +70,6 @@ CardStackRec	pileStacks[NUM_PILES];
 
 CardRec		rawcards[NUM_CARDS];
 
-CardStackPtr	fromStack;
-CardPtr		fromCard;
 int		dealNumber;
 
 typedef struct _klondikeResources {
@@ -215,8 +214,6 @@ NewGame (void)
     CardsRemoveAllCards (deck);
     CardsRemoveAllCards (piles);
     CardsRemoveAllCards (stacks);
-    fromStack = 0;
-    fromCard = 0;
     InitStacks ();
     GenerateCards ();
     CardShuffle (&deckStacks[0], False);
@@ -233,11 +230,13 @@ Undo (void)
     DisplayStacks ();
 }
 
+#define MAX_SCORE 340
+
 static void
 Score (void)
 {
-    Message (message, "Current position scores %d out of 340.",
-	     ComputeScore ());
+    Message (message, "Current position scores %d out of %d.",
+	     ComputeScore (), MAX_SCORE);
 }
 
 static void
@@ -307,19 +306,20 @@ static CardStackPtr
 FindRegularPlay (CardStackPtr from_stack, CardPtr *from_cardp)
 {
     int		    i;
-    int		    col = from_stack - stackStacks;
     CardStackPtr    to_stack;
     CardPtr	    from_card;
 
     if (*from_cardp)
 	from_card = *from_cardp;
-    else if (from_stack->last)
-	from_card = CardInReverseAlternatingSuitOrder (from_stack->last);
-    else
+    else if (from_stack->last) {
+        from_card = from_stack->last;
+        if (from_stack->widget == stacks)
+            from_card = CardInReverseAlternatingSuitOrder (from_card);
+    } else
 	return NULL;
     for (i = 0; i < NUM_STACKS; i++)
     {
-	if (i == col)
+	if (from_stack == &stackStacks[i])
 	    continue;
 	to_stack = &stackStacks[i];
 	if (IsLegalRegularPlay (from_stack, from_card, to_stack))
@@ -335,7 +335,8 @@ static Boolean
 IsLegalEmptyPlay (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
     (void) from_stack;
-    if (from_card->card.rank == CardsKing && to_stack->last == NULL)
+    (void) from_card;
+    if (to_stack->last == NULL)
 	return True;
     return False;
 }
@@ -434,6 +435,8 @@ Play (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 	}
     }
     CardMove (from_stack, from_card, to_stack, True);
+    if (ComputeScore() == MAX_SCORE)
+        Message(message, "We have a winner!");
 }
 
 static Boolean
@@ -586,98 +589,86 @@ Expand (CardStackPtr stack)
 
 /* Callbacks to user interface functions */
 
-static void
-StackCallback (Widget w, XtPointer closure, XtPointer data)
+static CardStackPtr
+WidgetToStack(Widget w, int col)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
-    CardPtr	    card;
-    String	    type;
-
-    (void) closure;
-    Message (message, "");
     if (w == stacks)
-	stack = &stackStacks[input->col];
-    else if (w == deck)
-	stack = &deckStacks[input->col];
-    else if (w == piles)
-	stack = &pileStacks[input->col];
-    else
-	return;
-    for (card = stack->last; card; card = card->prev)
-	if (card->isUp && card->row == input->row)
-	    break;
-    if (*input->num_params) {
-	type = *input->params;
-	if (!strcmp (type, "deck_source"))
-	{
-	    if (!stack->last)
-		Message (message, "Deck is empty");
-	    else
-	    {
-		fromStack = stack;
-		fromCard = stack->last;
-	    }
-	}
-	else if (!strcmp (type, "stack_source"))
-	{
-	    if (!stack->last)
-		Message (message, "Selected stack is empty.");
-	    else
-	    {
-		fromStack = stack;
-		fromCard = 0;
-	    }
-	}
-	else if (!strcmp (type, "card_source"))
-	{
-	    if (!card)
-		Message (message, "No card selected.");
-	    else
-	    {
-		fromStack = stack;
-		fromCard = card;
-	    }
-	}
-	else if (!strcmp (type, "dest"))
-	{
-	    if (fromStack)
-	    {
-		Play (fromStack, fromCard, stack);
-		CheckStackTop (fromStack);
-		fromStack = NULL;
-		CardNextHistory ();
-		DisplayStacks ();
-	    }
-	}
-	else if (!strcmp (type, "expand"))
-	{
-	    Expand (stack);
-	}
-    }
+	return &stackStacks[col];
+    if (w == piles)
+	return &pileStacks[col];
+    if (w == deck)
+	return &deckStacks[col];
+    return NULL;
 }
 
 static void
-DeckCallback (Widget w, XtPointer closure, XtPointer data)
+InputCallback (Widget w, XtPointer closure, XtPointer data)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
+    HandInputPtr    input = (HandInputPtr) data;
+    CardStackPtr    stack = NULL;
+    CardStackPtr    startStack = NULL;
 
+    (void) closure;
     Message (message, "");
-    stack = &deckStacks[input->col];
-    if (input->col == 0 && *input->num_params && !strcmp (*input->params, "deck_source"))
-    {
-	if (!stack->last)
-	    ResetDeck ();
-	else
-	    Deal ();
+    stack = WidgetToStack(w, input->current.col);
+    startStack = WidgetToStack(input->start.w, input->start.col);
+
+    if (!startStack || !stack)
+	return;
+
+    switch (input->action) {
+    case HandActionStart:
+        break;
+    case HandActionClick:
+        CardSetAnimate(True);
+        if (stack == &deckStacks[0]) {
+            if (!stack->last)
+                ResetDeck ();
+            else
+                Deal ();
+            CardNextHistory ();
+            DisplayStacks ();
+        }
+        else
+        {
+            Play (stack, NULL, stack);
+            CheckStackTop (stack);
+            CardNextHistory ();
+            DisplayStacks ();
+	}
+        break;
+    case HandActionDrag:
+        CardSetAnimate(False);
+        if (startStack == &deckStacks[0] && stack == &deckStacks[1]) {
+            if (startStack->last) {
+                Deal ();
+                CardNextHistory ();
+                DisplayStacks ();
+            }
+        }
+        else if (startStack == &deckStacks[1] && stack == &deckStacks[0])
+        {
+            if (!stack->last) {
+                ResetDeck ();
+                CardNextHistory ();
+                DisplayStacks ();
+            }
+        }
+        else if (startStack->last)
+        {
+            CardPtr     card = CardFromHandCard(input->start.private);
+            Play (startStack, card, stack);
+            CheckStackTop (startStack);
+            CardNextHistory ();
+            DisplayStacks ();
+	}
+        break;
+    case HandActionExpand:
+        Expand (stack);
+        break;
+    case HandActionUnexpand:
+        break;
     }
-    else
-    {
-	StackCallback (w, closure, data);
-    }
-    CardNextHistory ();
-    DisplayStacks ();
 }
 
 static void
@@ -855,11 +846,11 @@ CreateMenu (Widget parent, char *name, struct menuEntry *entries, int count)
     Widget  entry;
     int	    i;
 
-    menu = XtCreatePopupShell (name, simpleMenuWidgetClass,
+    menu = XtCreatePopupShell (name, ksimpleMenuWidgetClass,
 			       parent, NULL, ZERO);
     for (i = 0; i < count; i++) {
 	entry = XtCreateManagedWidget (entries[i].name,
-				       smeBSBObjectClass, menu, NULL, ZERO);
+				       ksmeBSBObjectClass, menu, NULL, ZERO);
 	XtAddCallback (entry, XtNcallback, entries[i].function, NULL);
     }
     return menu;
@@ -884,12 +875,8 @@ main (int argc, char **argv)
 {
     Atom wm_delete_window;
 
-#ifdef APPDEFAULTS
-    setenv("XAPPLRESDIR", APPDEFAULTS, 1);
-#endif
-
-    toplevel = XtInitialize (argv[0], "KKlondike", options, XtNumber(options),
-			     &argc, argv);
+    toplevel = XkwInitialize ("KKlondike", options, XtNumber(options),
+			      &argc, argv, True, defaultResources);
 
     XtGetApplicationResources (toplevel, (XtPointer)&klondikeResources, resources,
 			       XtNumber (resources), NULL, 0);
@@ -904,32 +891,32 @@ main (int argc, char **argv)
     frame = XtCreateManagedWidget ("frame", layoutWidgetClass, toplevel, NULL, 0);
     menuBar = XtCreateManagedWidget ("menuBar", layoutWidgetClass, frame, NULL, 0);
     fileMenuButton = XtCreateManagedWidget ("fileMenuButton",
-					    menuButtonWidgetClass,
+					    kmenuButtonWidgetClass,
 					    menuBar, NULL, ZERO);
     fileMenu = CreateMenu (fileMenuButton, "fileMenu",
 			   fileMenuEntries, XtNumber (fileMenuEntries));
-    newGame = XtCreateManagedWidget ("newGame", commandWidgetClass,
+    newGame = XtCreateManagedWidget ("newGame", kcommandWidgetClass,
 				     menuBar, NULL, ZERO);
     XtAddCallback(newGame, XtNcallback, NewGameCallback, NULL);
-    undo = XtCreateManagedWidget ("undo", commandWidgetClass,
+    undo = XtCreateManagedWidget ("undo", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(undo, XtNcallback, UndoCallback, NULL);
-    hint = XtCreateManagedWidget ("hint", commandWidgetClass,
+    hint = XtCreateManagedWidget ("hint", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(hint, XtNcallback, FindAMoveCallback, NULL);
-    score = XtCreateManagedWidget ("score", commandWidgetClass,
+    score = XtCreateManagedWidget ("score", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(score, XtNcallback, ScoreCallback, NULL);
-    pileAll = XtCreateManagedWidget ("pileAll", commandWidgetClass,
+    pileAll = XtCreateManagedWidget ("pileAll", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(pileAll, XtNcallback, PileAllCallback, NULL);
     deck = XtCreateManagedWidget ("deck", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (deck, XtNinputCallback, DeckCallback, NULL);
+    XtAddCallback (deck, XtNinputCallback, InputCallback, NULL);
     piles = XtCreateManagedWidget ("piles", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (piles, XtNinputCallback, StackCallback, NULL);
+    XtAddCallback (piles, XtNinputCallback, InputCallback, NULL);
     stacks = XtCreateManagedWidget ("stacks", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (stacks, XtNinputCallback, StackCallback, NULL);
-    message = XtCreateManagedWidget ("message", labelWidgetClass, frame, NULL, 0);
+    XtAddCallback (stacks, XtNinputCallback, InputCallback, NULL);
+    message = XtCreateManagedWidget ("message", klabelWidgetClass, frame, NULL, 0);
     srandom (getpid () ^ time ((long *) 0));
     NewGame ();
     XtRealizeWidget (toplevel);
@@ -938,5 +925,7 @@ main (int argc, char **argv)
     (void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
                             &wm_delete_window, 1);
 
+    XkwSetCardIcon(toplevel);
+
     XtMainLoop ();
 }
